//+------------------------------------------------------------------+
//|                                              MultixpMAEAV1.0.mq4 |
//| Designed and coded by Ralph Ronquist to version 1.0              |
//+------------------------------------------------------------------+


#property copyright ""
#property link      ""

//----------------------- INCLUDES
#include <stderror.mqh>

//----------------------- EA PARAMETER
extern string xpmafile = "xpma.csv";
string Expert_Name          = "MultixpMAEA";
int MagicNumber          = 12345;

extern string Order_Setting        = "---------- Order Setting";
extern bool OneEntryPerBar      = true;
extern bool ReverseTradesOnly   = false; // Avoid stuttering or not
extern double StopLoss   = 40;
extern double TakeProfit = 100;
extern int BreakEvenAfterPips = 40; 
extern int Slippage      = 5;
extern int MaxOpenTrade         = 1;
extern int MinPriceDistance     = 5;

extern string TrailingStop_Setting = "---------- Trail Management Settings";
extern int TrailingStopType = 1;
extern int TrailingStop     = 0;
        
extern string Moving_Average_Setting = "---------- MA close  Settings";
extern int CloseMA = 0;

extern string RSI_Settings         = "---------- RSI Filter Settings";
extern bool UseRSIfilter         = false;
extern int RSIPeriod            = 21;
extern int RSIPrice             = 1; 
extern double RSIBUYthreshold = 50.0;
extern double RSISELLthreshold = 50.0;

extern string CCI_Settings         = "---------- CCI Filter Settings";
extern bool UseCCIfilter         = false;
extern int CCIPeriod            = 21;
extern int CCIPrice             = 1; 
extern double CCIBUYthreshold = 50.0;
extern double CCISELLthreshold = -50.0;

extern string Volume_Settings         = "---------- Volume Filter Settings";
extern bool UseVolumeFilter         = false;
extern int VolumeRange = 1;
extern double VolumeThreshold = 7.0;


extern string Time_Parameters      = "---------- EA Active Time";
extern bool UseHourTrade         = false;         
extern int StartHour            = 10;
extern int EndHour              = 11;
         
extern string MM_Parameters        = "---------- Money Management";
extern double Lots                 = 1;
extern bool MM                   = true; //Use Money Management or not
extern bool AccountIsMicro       = false; //Use Micro-Account or not
extern double DecreaseFactor       = 1; //Decrease Lots After Loss
extern int Risk                 = 10; //10%
         
extern string Alert_Setting        = "---------- Alert Setting";
extern bool EnableAlert          = true;
extern bool EmailAlert          = false;
extern string SoundFilename        = "alert.wav";

extern string Testing_Parameters= "---------- Back Test Parameter";
extern bool PrintControl         = true;
extern bool Show_Settings        = true;

int NumberOfTries        = 10;
//----- Globals
string TicketComment;

//+------------------------------------------------------------------+
//| Representations for xpMA lines (their configurations and values) |
//+------------------------------------------------------------------+
    // readings
    #define XPMA_Value          0
    #define XPMA_Last_Signal    1
        #define UP_SIGNAL  1
        #define DN_SIGNAL -1
        #define NO_SIGNAL  0
    #define XPMA_Time           2
    // settings
    #define XPMA_TimeFrame      3
    #define XPMA_MA_Period      4
    #define XPMA_MA_Type        5
        //MODE_SMA           0
        //MODE_EMA           1
        //MODE_SMMA          2
        //MODE_LWMA          3
        #define MODE_DEMA    4
        #define MODE_TEMA    5
        #define MODE_T3MA    6
        #define MODE_JMA     7
        #define MODE_HMA     8
        #define MODE_DECEMA  9
        #define MODE_SALT    10
    #define XPMA_MA_Applied     6
        //PRICE_CLOSE    0
        //PRICE_OPEN     1
        //PRICE_HIGH     2
        //PRICE_LOW      3
        //PRICE_MEDIAN   4
        //PRICE_TYPICAL  5
        //PRICE_WEIGHTED 6
    #define XPMA_T3MA_VolumeFactor 7
    #define XPMA_JMA_Phase      8
    #define XPMA_Step_Period    9
    #define XPMA_Shift         10
    #define XPMA_Type          11
        //#define USE_XPMA_NOT 0
        #define USE_XPMA_LINE 1
        #define USE_XPMA_RELATIVE 2
    #define XPMA_Span          12
    
#define XPMA_WIDTH 13

double xpma[][XPMA_WIDTH];

//--- trade codes
#define BUY 1
#define SELL -1
//NULL 0

//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
{
    //----------------------- GENERATE MAGIC NUMBER AND TICKET COMMENT
    //----------------------- SOURCE : PENGIE
    MagicNumber    = subGenerateMagicNumber( MagicNumber, Symbol(), Period() );
	TicketComment  = StringConcatenate( "MultixpMAEA-", Symbol(), "-", Period());

    //----------------------- SHOW EA SETTING ON THE CHART
    //----------------------- SOURCE : CODERSGURU
    if ( Show_Settings )
        subPrintDetails();
   
    MaxOpenTrade = MathMax( MaxOpenTrade, 1 );
    loadXPMAConfigs();
    Print( "INITED" );
    return( 0 );
}

//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
{
   return(0);
}

//+------------------------------------------------------------------+
//| EA start; this method is invoked once for each "tick"             |
//+------------------------------------------------------------------+
int start()
{
    manageOpened(); // Happens at every tick.

    if ( Bars < 100 ) {
        return(0);  
    }

    if ( ! tradingHours() ) {
        Comment( "Not Trading Hours!" );
        return( 0 );
    }

    if ( ! timeToCheckIndicators() ) {
        return( 0 );
    }

    int trade = NULL;
    if ( ! entryCondition( trade ) ) {
        Comment( "No trade condition" );
        return( 0 );
    }

    if ( subTotalTrades() >= MaxOpenTrade )
        return( 0 );

    if ( ! filters( trade ) )
        return( 0 );
    
    subOpenOrder( trade, StopLoss, TakeProfit );

    notifications( trade );
    
    return( 0 );
}

//+------------------------------------------------------------------+
//| Utility functions....                                            |
//+------------------------------------------------------------------+

//-- Returns true if current time is a trading hour
bool tradingHours()
{
    if ( ! UseHourTrade )
        return( true );
    if ( StartHour <= EndHour )
        return( Hour() >= StartHour && Hour() <= EndHour );
    return ( Hour() >= StartHour || Hour() <= EndHour );
}

//-- Returns true if it is time to check indicators
bool timeToCheckIndicators(){
    static datetime last_check_time = 0;
    if ( ! OneEntryPerBar )
        return( true );
    if ( last_check_time == Time[0] )
        return ( false );
    last_check_time = Time[0];
    return ( true );
}

bool selectOrder(int pos)
{
    if ( ! OrderSelect( pos, SELECT_BY_POS ) )
        return( false );
    if ( OrderMagicNumber() != MagicNumber )
        return( false );
    return( OrderSymbol() == Symbol() );
}

//---------- Control function to manage opened trades
// Reviews order history, and invokes trailing-stop or break-even
// functions for trades to be managed by this EA.
void manageOpened()
{
    for ( int i = OrdersTotal() - 1; i >= 0; i-- ) {
        if ( ! selectOrder( i ) ) {
            continue;
        }
        if ( subMaybeCloseOrder() )
            continue;
        if ( TrailingStop > 0 ) {
            subTrailingStop();
        }
        if ( BreakEvenAfterPips > 0 ) {
            subBreakEvenAfterPips();
        }
    }
}

//-- returns number of xpma lines
int xpmaSize()
{
    return( ArraySize( xpma ) / XPMA_WIDTH );
}

//----
// Load xpma configurations from a file, which has 8 numbers on each line
// separated by comma (i.e., a CSV file). The configurations are in order:
//    TimeFrame, MA_Period, Step_Period, MA_Type, MA_Applied,
//    T3MA_Volume_Factor, JMA_Phase, shift, type and span.
void loadXPMAConfigs()
{
    int handle = FileOpen( xpmafile, FILE_CSV | FILE_READ, "," );
    if ( handle < 0 ) {
        int error = GetLastError();
        Print( "Opening " + xpmafile + " failed: " + error );
        return;
    }
    int TimeFrame = FileReadNumber( handle ); // TimeFrame
    while ( ! FileIsEnding( handle ) ) {
        int size = xpmaSize();
        int sx = ArrayResize( xpma, size + 1 );
        xpma[ size ][ XPMA_TimeFrame ] = TimeFrame;
        xpma[ size ][ XPMA_MA_Period ] = FileReadNumber( handle );
        xpma[ size ][ XPMA_MA_Type ] = FileReadNumber( handle );
        xpma[ size ][ XPMA_MA_Applied ] = FileReadNumber( handle );
        xpma[ size ][ XPMA_T3MA_VolumeFactor ] = FileReadNumber( handle );
        xpma[ size ][ XPMA_JMA_Phase ] = FileReadNumber( handle );
        xpma[ size ][ XPMA_Step_Period ] = FileReadNumber( handle );
        xpma[ size ][ XPMA_Shift ] = FileReadNumber( handle );
        xpma[ size ][ XPMA_Type ] = FileReadNumber( handle );
        xpma[ size ][ XPMA_Span ] = FileReadNumber( handle );
        TimeFrame = FileReadNumber( handle ); // TimeFrame
    }
    FileClose( handle );
}

//----
// Read off i:th xpMA line
// Sets [i][MA_Value] to the value, and [i][MA_Last_signal] to UP_SIGNAL (1)
// or DN_SIGNAL (-1) if the xpma has such a signal, otherwise [i][MA_Last_signal]
// is left unchanged
double readXPMA(int index,int buffer)
{
    int TimeFrame               = xpma[ index ][ XPMA_TimeFrame ];
    int MA_Period               = xpma[ index ][ XPMA_MA_Period ];
    int MA_Type                 = xpma[ index ][ XPMA_MA_Type ];
    int MA_Applied              = xpma[ index ][ XPMA_MA_Applied ];
    double T3MA_VolumeFactor    = xpma[ index ][ XPMA_T3MA_VolumeFactor ];
    double JMA_Phase            = xpma[ index ][ XPMA_JMA_Phase ];
    int Step_Period             = xpma[ index ][ XPMA_Step_Period ];
    int shift                   = xpma[ index ][ XPMA_Shift ];
    
    return( iCustom( NULL, 0, "xpMA", TimeFrame, MA_Period, MA_Type, MA_Applied,
        T3MA_VolumeFactor, JMA_Phase, Step_Period, false, buffer, shift ) );
}

#define RED_LINE 2
#define BLUE_LINE 1
#define LINE_VALUE 0

void xpmaSample(int index)
{
    string name = "xpMA(" + xpma[ index ][ XPMA_MA_Period ] + ")";
    
    xpma[ index ][ XPMA_Value ] = readXPMA( index, LINE_VALUE );

    if ( readXPMA( index, RED_LINE ) != EMPTY_VALUE ) {
        if ( xpma[ index ][ XPMA_Last_Signal ] != UP_SIGNAL )
            xpma[ index ][ XPMA_Time ] = TimeCurrent();
        xpma[ index ][ XPMA_Last_Signal ] = UP_SIGNAL;
    } else if ( readXPMA( index, BLUE_LINE ) != EMPTY_VALUE ) {
        if ( xpma[ index ][ XPMA_Last_Signal ] != DN_SIGNAL )
            xpma[ index ][ XPMA_Time ] = TimeCurrent();
        xpma[ index ][ XPMA_Last_Signal ] = DN_SIGNAL;
    }
}

//----
// Read off all signal lines using their configurations
void readIndicators()
{
    static datetime last_read = 0;
    if ( last_read == Time[0] )
        return; // Only read once for each bar
    last_read = Time[0];
    
    for ( int i = 0; i < xpmaSize(); i++ ) {
        xpmaSample( i );
    }
}

//----
// Review all xpma values to see if they agree or not.
// Returns UP_SIGNAL (1) or DN_SIGNAL (-1) if all agree on it, and
// NO_SIGNAL (0) otherwise.
// + A USE_XPMA_LINE type MA agrees by means of line direction as signified by
//   its color, but limited to the initial span of bars
// + A USE_XPMA_RELATIVE type MA agrees by means of its value being correctly
//   above or below previous USE_XPMA_RELATIVE type MA.
// + Any other type MA always agrees.
// Note: the MA should be ordered from fastest to slowest
int direction()
{
    int i;
    int common_signal = NO_SIGNAL;
    double v = 0;

    for ( i = 0; i < xpmaSize(); i++ ) {
        int type = xpma[i][ XPMA_Type ];
        switch ( type ) {
        case USE_XPMA_LINE:
            int signal = xpma[i][ XPMA_Last_Signal ];
            if ( ( signal == UP_SIGNAL && common_signal == DN_SIGNAL ) ||
                 ( signal == DN_SIGNAL && common_signal == UP_SIGNAL ) ||
                 ( signal == NO_SIGNAL ) ) {
                return( NO_SIGNAL );
            }
            if ( xpma[i][ XPMA_Span ] > 0 ) {
                int bars = ( TimeCurrent() - xpma[i][ XPMA_Span ] ) / Period();
                if ( xpma[i][ XPMA_Span ] >= bars )
                    return( NO_SIGNAL );
            }
            common_signal = signal;
            break;
        case USE_XPMA_RELATIVE:
            if ( v == 0 ) {
                v = xpma[i][ XPMA_Value ];
                break;
            }
            if ( xpma[i][ XPMA_Value ] < v ) { // Slower below faster means upish
                if ( common_signal == DN_SIGNAL ) // Disagreement
                    return( NO_SIGNAL );
                common_signal = UP_SIGNAL;
                v = xpma[i][ XPMA_Value ];
            } else if ( xpma[i][ XPMA_Value ] > v ) { // Slower above faster means downish
                if ( common_signal == UP_SIGNAL ) // Disagreement
                    return( NO_SIGNAL );
                common_signal = DN_SIGNAL;
                v = xpma[i][ XPMA_Value ];
            }
            break;
        default:
            break;
        }
    }
    
    return( common_signal );
}

//---------- Decision logic for opening position
// Returns true or false to mark whether a tradable condition occurred, and
// sets the trade parameter as BUY (1) or SELL (-1) to indicate buy or sell.
// The function first checks for signal agreement, then translates common
// UP_SIGNAL as BUY and common DN_SIGNAL as SELL, but avoid stuttering when
// common signals keep re-occurring.
bool entryCondition(int &trade)
{
    static int last_trade = NULL;
    trade = NULL;
    readIndicators();
    switch ( direction() ) {
    case UP_SIGNAL:
        if ( last_trade == BUY && ReverseTradesOnly )
            return( false ); // Avoid stuttering
        trade = BUY;
        break;
    case DN_SIGNAL:
        if ( last_trade == SELL && ReverseTradesOnly )
            return( false ); // Avoid stutttering
        trade = SELL;
        break;
    case NO_SIGNAL:
        return( false );
    }
    last_trade = trade;
    return( true );
}

//---------- Trade Notifications to user
void notifications(int trade)
{
    string action = "Buy";
    if ( trade == SELL )
        action = "Sell";
        
    if ( PrintControl ) {
        Print( "MA Cross " + action );
    }

    //----------------------- alert and sound
    if ( EnableAlert ) {
        Alert( action + " " + Symbol() );
        PlaySound( SoundFilename );
    }
    
    if ( EmailAlert ) {
        SendMail( Expert_Name + " says " + action + " " + Symbol(),
            action + " " + Symbol() + " @ " + Open[0] + " (bid)"
         );
    }
}

//+------------------------------------------------------------------+
//| ADDITIONAL FILTER                                                |
//+------------------------------------------------------------------+
bool filters(int &trade)
{
    //-- RSI FILTER SETTINGS                                              |
    if ( UseRSIfilter ) {
        double RSI = iRSI(NULL,0,RSIPeriod,RSIPrice,0);
        if ( trade == BUY && ! ( RSI > RSIBUYthreshold ) ) {
            trade = NULL;
            return( false );
        }
        if ( trade == SELL && ! ( RSI < RSISELLthreshold ) ) {
            trade = NULL;
            return( false );
        }
    }
   
    //-- CCI FILTER SETTINGS                                              |
    if ( UseCCIfilter ) {
        double CCI = iCCI(NULL,0,CCIPeriod,CCIPrice,0);
        if ( trade == BUY && ! ( CCI > CCIBUYthreshold ) ) {
            trade = NULL;
            return( false );
        }
        if ( trade == SELL && ! ( CCI < CCISELLthreshold ) ) {
            trade = NULL;
            return( false );
        }
    }

    //-- Volume filter
    if ( UseVolumeFilter ) {
        double v = 0;
        for ( int i = 1; i <= VolumeRange; i++ ) {
            v += Volume[ i ];
        }
        if ( v < VolumeThreshold ) {
            return( false );
        }
    }
    return( true );
}

//----------------------- MONEY MANAGEMENT FUNCTION  
//----------------------- SOURCE : CODERSGURU
//----------------------- Adaption: Ralph Ronnquist
/**
 * Utility method to count how many lost trade of the symbol it has been
 * since last winner, if any.
 */
int reduction()
{
    int losses = 0;

    // Assumes that the history keeps trades in time order
    for(int i = OrdersHistoryTotal() - 1; i >= 0; i-- ) {
        if ( ! OrderSelect( i, SELECT_BY_POS, MODE_HISTORY ) )
            continue; // ignore
        if ( OrderSymbol() != Symbol() )
            continue;
        if ( OrderType() != OP_BUY && OrderType() != OP_SELL )
            continue;
         //----
        if ( OrderProfit() > 0 )
            break; // Found a winner
        losses++;
    }
    return( losses );
}

/**
 * Determines a good volume to trade with, given the current free margin, the
 * configured risk, and how many trades of this symbol has been lost.
 */
double subLotSize()
{
    if ( ! MM )
        return( Lots );

    double lot = AccountFreeMargin() * Risk / 100000.0;

    if ( DecreaseFactor > 0 ) {
        lot *= 1 - reduction() / DecreaseFactor;
    }

    if ( AccountIsMicro ) {
        lot = MathMin( 99, MathMax( 0.01, NormalizeDouble( lot, 2 ) ) );
    } else {
        lot = MathMin( 99, MathMax( 0.1, NormalizeDouble( lot, 1 ) ) );
    }
    return( lot );
}

//----------------------- NUMBER OF ORDER BASE ON SYMBOL AND MAGICNUMBER FUNCTION
int subTotalTrades()
{
    int count = 0;
    for ( int i = OrdersTotal() - 1; i >= 0; i-- ) {
        if ( selectOrder( i ) )
            count++;
    }
    return( count );
}


//----------------------- OPEN ORDER FUNCTION
void subOpenOrder(int trade, int stoploss, int takeprofit)
{
    int ticket = -1;
    int cmd;
    double price = 0;
    double stop = 0;
    double take = 0;

    // The very first trade is a start-up anomaly, so let's drop it
    static bool very_first = true;
    if ( very_first ) {
        very_first = false;
        return;
    }

    for(int c=0; c < NumberOfTries; c++ ) {
        double volume = subLotSize();
        if ( trade == BUY ) {
            cmd = OP_BUY;
            price = Ask;
            if ( stoploss > 0 )
                stop = NormalizeDouble( Ask - stoploss * Point, Digits );
            if ( takeprofit > 0 )
                take = NormalizeDouble( Ask + takeprofit * Point, Digits );
        } else if ( trade == SELL ) {
            cmd = OP_SELL;
            price = Bid;
            if ( stoploss > 0 )
                stop = NormalizeDouble( Bid + stoploss * Point, Digits );
            if ( takeprofit > 0 )
                take = NormalizeDouble( Bid - takeprofit * Point, Digits );
        } else {
            break;
        }
        price = NormalizeDouble( price, Digits );
        ticket = OrderSend( Symbol(), cmd, volume, price, Slippage, stop, take, TicketComment, MagicNumber );
        if ( ticket > 0 )
            break;
        int error = GetLastError();
        if ( retryError( error ) ) {
            Sleep( 5000 );
            RefreshRates();
            continue;
        }
        Print( "Unrecoverable open error: " + error );
        break;
    }
}

bool retryError(int error)
{
    static int retry_errors[] = {
        ERR_SERVER_BUSY,
        ERR_NO_CONNECTION,
        ERR_TOO_FREQUENT_REQUESTS,
        ERR_TRADE_TIMEOUT,
        ERR_PRICE_CHANGED,
        ERR_OFF_QUOTES,
        ERR_REQUOTE,
        ERR_TOO_MANY_REQUESTS,
        142, 143, 144,
        ERR_TRADE_CONTEXT_BUSY
    };
    for ( int i = 0; i < ArraySize( retry_errors ); i++ ) {
        if ( error == retry_errors[i] )
            return( true );
    }
    return( false );
}

/**
 * Check for closing signal on closeMA according to its type. Type 0 and 1 have
 * closing signal as per XPMA_Last_Signal.  Type 2 is special, with an attempt
 * to use its value crossing with the next, higher period MA (regardless of its
 * type). [CloseMA] > [CloseMA+1] means up and [CloseMA] < [CloseMA+1] means
 * down. If [CloseMA] == [CloseMA+1], then the XPMA_Last_Signal of CloseMA is
 * used.
 */
int closeMASignal()
{
    if ( xpma[ CloseMA ][ XPMA_Type ] == USE_XPMA_RELATIVE ) {
        if ( CloseMA < xpmaSize() - 1 ) {
            if ( xpma[ CloseMA ][ XPMA_Value ] > xpma[ CloseMA+1 ][ XPMA_Value ] )
                return( UP_SIGNAL );
            if ( xpma[ CloseMA ][ XPMA_Value ] < xpma[ CloseMA+1 ][ XPMA_Value ] )
                return( DN_SIGNAL );
        }
    }
    return( xpma[ CloseMA ][ XPMA_Last_Signal ] );
}

//----------------------- CLOSE ORDER FUNCTION
/**
 * Apply the nominated CloseMA to possibly close the selected trade.
 */
bool subMaybeCloseOrder()
{
    if ( CloseMA < 0 || CloseMA >= xpmaSize() )
        return( false );
    readIndicators();
    int signal = closeMASignal();
    int period = xpma[ CloseMA ][ XPMA_MA_Period ];

    if ( signal == 0 )
        return( false );
        
    switch ( OrderType() ) {
    case OP_BUY:
        if ( signal != UP_SIGNAL ) {
            Print( "Close BUY by MA" + CloseMA + " xpMA(" + period + ") signal " + signal );
            return( subCloseOrder() );
            
        }
        break;
    case OP_SELL:
        if ( signal != DN_SIGNAL ) {
            Print( "Close SELL by MA" + CloseMA + " xpMA(" + period + ") signal " + signal );
            return( subCloseOrder() );
        }
        break;
    }
    return( false );
}

bool subCloseOrder()
{
    int type = OrderType();
    
    if ( type != OP_BUY && type != OP_SELL ) {
        OrderDelete( OrderTicket() );
        return( true );
    }
    for ( int c = 0; c < NumberOfTries; c++ ) {
        double price = Bid;
        if ( type == OP_SELL )
            price = Ask;
        price = NormalizeDouble( price, Digits );
        if ( OrderClose( OrderTicket(), OrderLots(), price, Slippage, Violet ) )
            return( true );
        int error = GetLastError();
        if( retryError( error ) ) {
            Sleep(5000);
            RefreshRates();
            continue;
        }
        Print( "Unrecoverable close error: " + error );
        break;
    }
    return( false );
}
//****/

//----------------------- TRAILING STOP FUNCTION
// Uses interactive configurations:
//   TrailingStop = the gap from the price to maintain
//   TrailingStopType = 1 (default) to start trailing at 1*TrailingStop profit,
//                      2 to start at open price, or
//                      3 to limit by high/low of n:th prior candle
void subTrailingStop()
{
    setTrailingStop(
        trailingStopStart( TrailingStopType ),
        trailingStopGap( TrailingStopType )
    );
}

double trailingStopStart(int type)
{
    switch ( type ) {
    case 1: return ( TrailingStop * Point );
    case 2: return ( 0 );
    case 3: return ( 0 );
    default: return ( trailingStopStart( 1 ) );
    }
}

double trailingStopGap(int type)
{
    switch ( type ) {
    case 1: return ( TrailingStop * Point );
    case 2: return ( TrailingStop * Point );
    case 3:
        if ( TrailingStop <= 0 )
            return( 0 );
        if ( OrderType() == OP_BUY )
            return( Bid - Low[ TrailingStop ] );
        if ( OrderType() == OP_SELL )
            return( High[ TrailingStop ] - Ask );
        return( 0 );
    default:
        return ( trailingStopGap( 1 ) );
    }
}

void setTrailingStop(double start,double gap)
{
    double price;
    int type = OrderType();
    double stop = OrderStopLoss();
    if ( stop == 0 )
        stop = OrderOpenPrice();
    if ( type == OP_BUY ) {
        Print( "Trail? " + Bid + "," + stop + "+" + gap + "," + ( OrderOpenPrice() + start ) );
        if ( Bid > MathMax( stop + gap, OrderOpenPrice() + start ) ) {
            price = NormalizeDouble( Bid - gap, Digits );
            OrderModify( OrderTicket(), OrderOpenPrice(), price, OrderTakeProfit(), 0, Green );
        }
    } else if ( type == OP_SELL ) {
        if ( Ask < MathMin( stop - gap, OrderOpenPrice() - start ) ) {
            price = NormalizeDouble( Ask + gap, Digits );
            OrderModify( OrderTicket(), OrderOpenPrice(), price, OrderTakeProfit(), 0, Red );
        }
    }
}

//---------------------- BREAK EVEN FUNCTION
// Uses interactive configurations:
//    BreakEvenAfterPips = price gap required

void subBreakEvenAfterPips()
{
    if ( OrderType() == OP_BUY ) {
        if ( OrderStopLoss() >= OrderOpenPrice() - Point )
            return;
        if ( Bid <= OrderOpenPrice() + Point * BreakEvenAfterPips )
            return;
        OrderModify( OrderTicket(), OrderOpenPrice(), OrderOpenPrice(), OrderTakeProfit(), 0, Green );
    } else if ( OrderType() == OP_SELL ) {
        if ( OrderStopLoss() > 0 && OrderStopLoss() <= OrderOpenPrice() + Point )
            return;
        if ( Ask >= OrderOpenPrice() - Point * BreakEvenAfterPips )
            return;
        OrderModify( OrderTicket(), OrderOpenPrice(), OrderOpenPrice(), OrderTakeProfit(), 0, Red );
    }
}


//----------------------- GENERATE MAGIC NUMBER BASE ON SYMBOL AND TIME FRAME FUNCTION
//----------------------- SOURCE   : PENGIE
//----------------------- MODIFIED : FIREDAVE
int subGenerateMagicNumber(int MagicNumber, string symbol, int timeFrame)
{
   int isymbol = 0;
   if (symbol == "EURUSD")       isymbol = 1;
   else if (symbol == "GBPUSD")  isymbol = 2;
   else if (symbol == "USDJPY")  isymbol = 3;
   else if (symbol == "USDCHF")  isymbol = 4;
   else if (symbol == "AUDUSD")  isymbol = 5;
   else if (symbol == "USDCAD")  isymbol = 6;
   else if (symbol == "EURGBP")  isymbol = 7;
   else if (symbol == "EURJPY")  isymbol = 8;
   else if (symbol == "EURCHF")  isymbol = 9;
   else if (symbol == "EURAUD")  isymbol = 10;
   else if (symbol == "EURCAD")  isymbol = 11;
   else if (symbol == "GBPUSD")  isymbol = 12;
   else if (symbol == "GBPJPY")  isymbol = 13;
   else if (symbol == "GBPCHF")  isymbol = 14;
   else if (symbol == "GBPAUD")  isymbol = 15;
   else if (symbol == "GBPCAD")  isymbol = 16;
   else                          isymbol = 17;
   if(isymbol<10) MagicNumber = MagicNumber * 10;
   return (StrToInteger(StringConcatenate(MagicNumber, isymbol, timeFrame)));
}


//----------------------- PRINT COMMENT FUNCTION
//----------------------- SOURCE : CODERSGURU
void subPrintDetails()
{
   string sComment   = "";
   string sp         = "----------------------------------------\n";
   string NL         = "\n";

   sComment = sp;
   sComment = sComment + "TakeProfit=" + DoubleToStr(TakeProfit,0) + " | ";
   sComment = sComment + "TrailingStop=" + DoubleToStr(TrailingStop,0) + " | ";
   sComment = sComment + "StopLoss=" + DoubleToStr(StopLoss,0) + NL; 
   sComment = sComment + sp;
   sComment = sComment + sp;
   sComment = sComment + "Lots=" + DoubleToStr(Lots,2) + " | ";
   sComment = sComment + "MM=" + subBoolToStr(MM) + " | ";
   sComment = sComment + "Risk=" + DoubleToStr(Risk,0) + "%" + NL;
   sComment = sComment + sp;
  
   Comment(sComment);
}


//----------------------- BOOLEN VARIABLE TO STRING FUNCTION
//----------------------- SOURCE : CODERSGURU
string subBoolToStr ( bool value)
{
   if(value) return ("True");
   else return ("False");
}

//----------------------- END FUNCTION

